# -*- coding: utf-8 -*-

from xml.sax import saxutils
import weakref, re, sys


def quoteattr(value):
    '''Works like quoteattr from saxutils (returns escaped string in quotes),
    but is safe for HTML'''
    if value == '':
        return '""'
    return '"%s"' % saxutils.escape(unicode(value), {'"': '&quot;'})

def quoteattrs(data):
    '''Takes dict of attributes and returns their HTML representation'''
    items = []
    for key, value in data.items():
        items.append('%s=%s' % (key, quoteattr(value)))
    return ' '.join(items)

def quote_js(text):
    '''Quotes text to be used as JavaScript string in HTML templates. The
    result doesn't contain surrounding quotes.'''
    text = unicode(text) # for Jinja2 Markup
    text = text.replace('\\', '\\\\');
    text = text.replace('\n', '\\n');
    text = text.replace('\r', '');
    for char in '\'"<>&':
        text = text.replace(char, '\\x%2x' % ord(char))
    return text

def weakproxy(obj):
    '''Safe version of weakref.proxy.'''
    try:
        obj = weakref.proxy(obj)
    except TypeError:
        pass
    return obj


class cached_property(object):
    '''Turns decorated method into caching property (method is called once on
    first access to property).'''

    def __init__(self, method, name=None):
        self.method = method
        self.name = name or method.__name__
        self.__doc__ = method.__doc__

    def __get__(self, inst, cls):
        if inst is None:
            return self
        result = self.method(inst)
        setattr(inst, self.name, result)
        return result


class cached_class_property(object):
    '''Turns decorated method into caching class property (method is called
    once on first access to property of class or any of its instances).'''

    def __init__(self, method, name=None):
        self.method = method
        self.name = name or method.__name__

    def __get__(self, inst, cls):
        result = self.method(cls)
        setattr(cls, self.name, result)
        return result


def import_string(module_name, item_name=None):
    if item_name is None:
        return __import__(module_name, None, None, ['*'])
    return getattr(__import__(module_name, None, None, ['*']), item_name)


# http://www.w3.org/TR/REC-xml/#NT-Char
# Char ::= #x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD] | 
#          [#x10000- #x10FFFF]
# (any Unicode character, excluding the surrogate blocks, FFFE, and FFFF)
_char_tail = ''
if sys.maxunicode > 0x10000:
    _char_tail = u'%s-%s' % (unichr(0x10000),
                             unichr(min(sys.maxunicode, 0x10FFFF)))
_nontext_sub = re.compile(
                ur'[^\x09\x0A\x0D\x20-\uD7FF\uE000-\uFFFD%s]' % _char_tail,
                re.U).sub
def replace_nontext(text, replacement=u'\uFFFD'):
    return _nontext_sub(replacement, text)

def conf_to_dict(cfg):
    '''Selects all module members which names are upper case and return dict'''
    keys = filter(lambda x: x.isupper(), dir(cfg))
    d = {}
    for key in keys:
        d[key] = getattr(cfg, key)
    return d


# i18n markers
def N_(msg):
    '''Single translatable string marker'''
    return msg


class M_(object):
    '''Marker for translatable string with plural form'''
    def __init__(self, single, plural, count_field='count', format_args=None):
        self.single = single
        self.plural = plural
        self.count_field = count_field
        self.format_args = format_args

    def __mod__(self, format_args):
        return self.__class__(self.single, self.plural, count_field=self.count_field,
                              format_args=format_args)

    @property
    def count(self):
        args = self.format_args
        if args is None or \
                (isinstance(args, dict) and self.count_field not in args):
            raise TypeError("count is required")
        return args[self.count_field] if isinstance(args, dict) else args

    def __unicode__(self):
        args = self.format_args
        if self.count == 1:
            return self.single % args
        return self.plural % args








# -*- coding: utf-8 -*-

from urlparse import urlsplit
from lxml import html
from lxml.html import clean

# XXX move to iktomi.cms?

class Cleaner(clean.Cleaner):

    safe_attrs_only = True
    remove_unknown_tags = None
    drop_empty_tags = frozenset()
    allow_classes = None
    dom_callbacks = []
    allow_external_src = False
    allowed_protocols = frozenset(['http', 'https', 'mailto'])
    # None to allow all classes
    allow_classes = {}
    attr_val_is_uri = ['href', 'src', 'cite', 'action', 'longdesc']
    a_without_href = True

    def __call__(self, doc):
        clean.Cleaner.__call__(self, doc)
        if hasattr(doc, 'getroot'):
            # ElementTree instance, instead of an element
            doc = doc.getroot()
        self.extra_clean(doc)

    def extra_clean(self, doc):
        for el in doc.xpath('//*[@href]'):
            scheme, netloc, path, query, fragment = urlsplit(el.attrib['href'])
            if scheme and scheme not in self.allowed_protocols:
                el.drop_tag()

        for attr in self.attr_val_is_uri:
            if attr == 'href':
                continue
            for el in doc.xpath('//*[@'+attr+']'):
                scheme, netloc, path, query, fragment = urlsplit(el.attrib[attr])
                scheme_fail = scheme and scheme not in self.allowed_protocols
                netloc_fail = not self.allow_external_src and netloc
                if scheme_fail or netloc_fail:
                    if attr == 'src':
                        el.drop_tag()
                    else:
                        el.attrib.pop(attr)


        if self.a_without_href:
            for link in doc.xpath('//a[not(@href)]'):
                link.drop_tag()

        for tag in self.drop_empty_tags:
            for el in doc.xpath('//'+tag+'[not(./*)]'):
                has_text = el.text and el.text.strip(u'  \t\r\n\v\f\u00a0')
                if not el.attrib and not has_text:
                    el.drop_tag()

        if self.allow_classes is not None:
            for el in doc.xpath('//*[@class]'):
                classes = filter(None, el.attrib['class'].split())
                if el.tag in self.allow_classes:
                    allowed = self.allow_classes[el.tag]
                    condition = allowed if callable(allowed) else \
                            (lambda cls: cls in allowed)
                    classes = filter(condition, classes)
                else:
                    classes = []

                if classes:
                    el.attrib['class'] = ' '.join(classes)
                else:
                    el.attrib.pop('class')


        for callback in self.dom_callbacks:
            callback(doc)


def sanitize(value, **kwargs):
    doc = html.fragment_fromstring(value, create_parent=True)
    Cleaner(**kwargs)(doc)
    clean = html.tostring(doc, encoding='utf-8').decode('utf-8')
    return clean.split('>', 1)[1].rsplit('<', 1)[0]



# -*- coding: utf-8 -*-

import logging
from sqlalchemy import orm, create_engine
from sqlalchemy.orm.query import Query
from iktomi.utils import import_string
from iktomi.utils.deprecation import deprecated


class DBSession(orm.session.Session):

    @deprecated('Use Session.query(cls).filter_by(…).scalar() instead.')
    def get(self, query, **kwargs):
        if not isinstance(query, Query):
            query = self.query(query)
        if kwargs:
            query = query.filter_by(**kwargs)
        return query.first()


def multidb_binds(databases, package=None, engine_params=None):
    '''Creates dictionary to be passed as `binds` parameter to
    `sqlalchemy.orm.sessionmaker()` from dictionary mapping models module name
    to connection URI that should be used for these models. Models module must
    have `metadata` attribute. `package` when set must be a package or package
    name for all models modules.'''
    engine_params = engine_params or {}
    if not (package is None or isinstance(package, basestring)):
        package = getattr(package, '__package__', None) or package.__name__
    binds = {}
    for ref, uri in databases.items():
        md_ref = '.'.join(filter(None, [package, ref]))
        metadata = import_string(md_ref, 'metadata')
        engine = create_engine(uri, **engine_params)
        # Dot before [name] is required to allow setting logging level etc. for
        # all them at once.
        engine.logger = logging.getLogger('sqlalchemy.engine.[%s]' % ref)
        for table in metadata.sorted_tables:
            binds[table] = engine
    return binds


@deprecated('Use sqlalchemy.orm.sessionmaker(binds=multidb_binds(…)) instead.')
def session_maker(databases, query_cls=Query, models_location='models',
                  engine_params=None, session_params=None,
                  session_class=orm.session.Session):
    '''
    Session maker with multiple databases support. For each database there
    should be corresponding submodule of `models_location` package with
    `metadata` object for that database.
    '''
    engine_params = engine_params or {}
    session_params = dict(session_params or {})
    session_params.setdefault('autoflush', False)
    if isinstance(databases, basestring):
        engine = create_engine(databases, **engine_params)
        return orm.sessionmaker(class_=session_class, query_cls=query_cls,
                                bind=engine, **session_params)
    binds = multidb_binds(databases, models_location, engine_params=engine_params)
    return orm.sessionmaker(class_=session_class, query_cls=query_cls,
                            binds=binds, **session_params)



try:
    import Image
except ImportError:       # pragma: no cover
    from PIL import Image # pragma: no cover


class Resizer(object):

    def __init__(self, expand=False, filter=Image.ANTIALIAS):
        self.expand = expand
        self.filter = filter

    def transformations(self, img, target_size):
        # should return a list of JSON-serializable commands
        # that are processed with transform method
        raise NotImplementedError

    def transform(self, img, transformation, params):
        # transformations MUST be idempotent
        if transformation == 'crop':
            return img.crop(params)
        elif transformation == 'resize':
            return img.resize(params, self.filter)
        else:
            raise NotImplementedError(transformation)

    def __call__(self, img, target_size):
        transforms = self.transformations(img, target_size)
        for transformation, params in transforms:
            img = self.transform(img, transformation, params)
        return img


class ResizeFit(Resizer):

    def transformations(self, img, target_size):
        sw, sh = img.size
        tw, th = target_size
        if not self.expand and sw<=tw and sh<=th:
            return []

        if sw*th>sh*tw:
            h = sh*tw//sw
            return [('resize', (tw, h))]
        else:
            w = sw*th//sh
            return [('resize', (w, th))]


class ResizeCrop(Resizer):

    def transformations(self, img, target_size):
        sw, sh = img.size
        tw, th = target_size
        if not self.expand and sw<=tw and sh<=th:
            return []

        transforms = []
        if sw*th>sh*tw:
            if sh!=th and (sh>th or self.expand):
                w = sw*th//sh
                transforms.append(('resize', (w, th)))
                sw, sh = w, th
            if sw>tw:
                wd = (sw-tw)//2
                transforms.append(('crop', (wd, 0, tw+wd, sh)))
        else:
            if sw!=tw and (sw>tw or self.expand):
                h = sh*tw//sw
                transforms.append(('resize', (tw, h)))
                sw, sh = tw, h
            if sh>th:
                hd = (sh-th)//2
                transforms.append(('crop', (0, hd, sw, th+hd)))
        return transforms


class ResizeMixed(Resizer):

    def __init__(self, hor_resize, vert_resize):
        self.hor_resize = hor_resize
        self.vert_resize = vert_resize

    def get_resizer(self, img, target_size):
        sw, sh = img.size
        if sw >= sh:
            return self.hor_resize
        else:
            return self.vert_resize

    def transformations(self, img, size):
        return self.get_resizer(img, size).transformations(img, size)

    #def transform(self, img, *args):
    # XXX is this method needed?
    #    return self.get_resizer(img).transform(img, *args)

    def __call__(self, img, size):
        return self.get_resizer(img, size)(img, size)


class ResizeFixedWidth(Resizer):
    def transformations(self, img, target_size):
        sw, sh = img.size
        tw, th = target_size
        if not self.expand and sw<=tw:
            return []
        h = sh*tw//sw
        return [('resize', (tw, h))]


class ResizeFixedHeight(Resizer):

    def transformations(self, img, target_size):
        sw, sh = img.size
        tw, th = target_size
        if not self.expand and  sh<=th:
            return []
        w = sw*th//sh
        return [('resize', (w, th))]



# -*- coding: utf-8 -*-
""" New implementation of cache handler.
XXX: Before marge to cache.py lets test as separate module for
     a while for testing and fast rollback

Supports two types of cache:
* CacheManager class is for basic cache that stores only content
* CacheManagerWithContentType is for cache with content type (as separate cache key)
"""

__all__ = ['CacheManager', 'CacheManagerWithContentType', 'cache', 'nocache']

from iktomi import web
import logging


logger = logging.getLogger(__name__)
logger.setLevel(logging.WARNING)  # Set to INFO explicitly to see messages


def nocache(handler):
    def wrap(*args, **kwargs):
        response = handler(*args, **kwargs)
        setattr(response, '_CACHE_ENABLED', False)
        return response
    return wrap


def cache(duration=None, content_type='text/html'):
    def decor(handler):
        def wrap(*args, **kwargs):
            response = handler(*args, **kwargs)
            setattr(response, '_CACHE_ENABLED', True)
            setattr(response, '_CACHE_DURATION', duration)
            setattr(response, '_CACHE_CONTENT_TYPE', content_type)
            return response
        return wrap
    return decor


class NocacheHandler(web.WebHandler):
    def __call__(self, env, data):
        return nocache(self.next_handler)(env, data)


class CacheHandler(web.WebHandler):
    def __init__(self, duration, content_type='text/html'):
        self._duration = duration
        self._content_type = content_type

    def __call__(self, env, data):
        return cache(self._duration, self._content_type)(self.next_handler)(env, data)


class CacheManager(web.WebHandler):
    _cache_name_length = 250

    def __init__(self, storage, default_duration, durations, content_type='text/html'):
        self._storage = storage
        self._default_duration = default_duration
        self._durations = durations
        self._content_type = content_type


    def get_cache_name(self, env):
        cache_name = env.request.url
        if len(cache_name) > self._cache_name_length:
            return None
        return cache_name

    def get_duration(self, response):
        if not getattr(response, '_CACHE_ENABLED', True):
            return None

        duration = getattr(response, '_CACHE_DURATION', self._default_duration)
        if isinstance(duration, int):
            return duration
        if duration in self._durations:
            return self._durations[duration]
        return self._default_duration

    def save_response_to_cache(self, env, response, duration):
        """ Save response to cache without guaranty
        """
        content = response.body
        cache_name = self.get_cache_name(env)
        if cache_name is None or duration is None:
            return None
        logger.info('Caching for %i seconds %s', duration, cache_name)
        if isinstance(content, unicode):
            content = content.encode('utf-8')
        self._storage.set(cache_name, content, time=duration)

    def get_response_from_cache(self, env):
        cache_name = self.get_cache_name(env)
        if cache_name is None:
            return None
        body = self._storage.get(cache_name)
        logger.info('Getting from cache. Got %s',
                    (body if body is None else type(body)))
        if body is None:
            return None
        logger.info('Got from cache by `%s`', cache_name)
        return web.Response(body, content_type=self._content_type)

    def call_view(self, env, data):
        logger.info("Call view %s", env.request.url)
        response = self.next_handler(env, data)
        if response is not None:
            self.save_response_to_cache(env, response, self.get_duration(response))
            return response
        return None

    def __call__(self, env, data):
        if env.request.method not in ('GET', 'HEAD'):
            response = self.next_handler(env, data)
            logger.info("Cache skipped")
            return response
        
        cached = self.get_response_from_cache(env)
        if not cached:
            cached = self.call_view(env, data)
        return cached


    def cache(self, duration=None, content_type=None):
        if duration is None:
            duration = self._default_duration
        if content_type is None:
            content_type = self._content_type
        return CacheHandler(duration, content_type)

    def nocache(self):
        return NocacheHandler()


class CacheManagerWithContentType(CacheManager):
    _cache_name_length = 247

    def __init__(self, storage, default_duration, durations, content_type='text/html', content_type_prefix='ct:'):
        super(CacheManagerWithContentType, self).__init__(storage, default_duration, durations, content_type)
        self._ct_prefix = 'ct:'

    def save_response_to_cache(self, env, response, duration):
        super(CacheManagerWithContentType, self)\
            .save_response_to_cache(env, response, duration)
        
        content_type = response.headers.getall('content-type')
        if not content_type:
            content_type = self._content_type
        else:
            # XXX should we check if there is only one value as it done in WebOb?
            content_type = content_type[0]
        logger.info('Caching Content-Type `%s` for %i seconds %s', content_type,
                    duration, cache_name)
        self._storage.set(self._ct_prefix + cache_name, content_type, duration)


    def get_response_from_cache(self, env):
        response = super(CacheManagerWithContentType, self)\
            .get_response_from_cache(env)
        if response is None:
            return None
        content_type = self._storage.get(self._ct_prefix + cache_name)
        logger.info('Got from cache: %s , Content-Type: %s',
                    (body if body is None else type(body)), content_type)
        if content_type is None:
            return None
        response.content_type = content_type
        return response




# -*- coding: utf-8 -*-

class StorageFrame(object):

    def __init__(self, _parent_storage=None, **kwargs):
        self._parent_storage = _parent_storage
        self.__dict__.update(kwargs)

    def as_dict(self):
        d = dict(self._parent_storage.as_dict() if self._parent_storage else {},
                 **self.__dict__)
        # XXX delete _parent_storage?
        del d['_parent_storage']
        if '_root_storage' in d:
            del d['_root_storage']
        return d

class VersionedStorage(object):

    def __init__(self, cls=StorageFrame, *args, **kwargs):
        kwargs['_root_storage'] = self
        self._storage = cls(*args, **kwargs)

    def _push(self, **kwargs):
        self._storage = StorageFrame(_parent_storage=self._storage, **kwargs)
        return self._storage

    def _pop(self):
        self._storage = self._storage._parent_storage

    def __getattr__(self, name):
        frame = self._storage
        while frame:
            try:
                return getattr(frame, name)
            except AttributeError:
                frame = frame._parent_storage
        #raise AttributeError(name)
        raise AttributeError("{} has no attribute {}".format(
                             self.__class__.__name__, name))

    def __setattr__(self, name, value):
        if name == '_storage':
            self.__dict__[name] = value
        else:
            setattr(self._storage, name, value)

    def __delattr__(self, name):
        delattr(self._storage, name)

    def as_dict(self):
        return self._storage.as_dict()


class storage_property_base(object):

    def __init__(self, method, name=None):
        self.method = method
        self.name = name or method.__name__
        self.__doc__ = method.__doc__


class storage_property(storage_property_base):
    '''Turns decorated method into storage property (method is called with
       VersionedStorage as self).'''

    def __get__(self, inst, cls):
        if inst is None:
            return self
        return self.method(inst._root_storage)


class storage_cached_property(storage_property_base):
    '''Turns decorated method into storage cached property 
       (method is called only once with VersionedStorage as self).'''

    def __get__(self, inst, cls):
        if inst is None:
            return self
        result = self.method(inst._root_storage)
        setattr(inst, self.name, result)
        return result

def storage_method(func):
    '''Calls decorated method with VersionedStorage as self'''
    def wrap(self, *args, **kwargs):
        return func(self._root_storage, *args, **kwargs)
    return wrap



# -*- coding: utf-8 -*-
'''
Module containing some predefined form converters - objects designed to
convert and validate form field's data.
'''

import re
from ..utils import weakproxy, replace_nontext
from datetime import datetime
from collections import OrderedDict
from ..utils.dt import strftime
from ..utils.deprecation import deprecated
try:
    from ..utils.html import Cleaner
    from lxml import html
    from lxml.etree import XMLSyntaxError
except ImportError:
    # lxml is required for Html conv, therefore you can use forms without this
    # functionality
    Cleaner = None

from ..utils import N_, M_, cached_property

_all2 = locals().keys()


class ValidationError(Exception):

    def __init__(self, message=None, by_field=None):
        self.message = message
        self.by_field = by_field or {}

    def translate(self, env, message):
        if isinstance(message, M_):
            trans = env.ngettext(unicode(message.single),
                                 unicode(message.plural),
                                 message.count)
            return trans % message.format_args
        return env.gettext(message)

    def fill_errors(self, field):
        form = field.form
        if self.message is not None:
            form.errors[field.input_name] = self.translate(form.env, self.message)
        for name, message in self.by_field.items():
            if name.startswith('.'):
                nm, f = name.lstrip('.'), field
                for i in xrange(len(name) - len(nm) - 1):
                    f = f.parent
                name = f.get_field(nm).input_name
            form.errors[name] = self.translate(form.env, message)

    def __repr__(self):
        return "%s(%r, %r)" % (self.__class__,
                               self.message,
                               self.by_field)


class Converter(object):
    '''
    :meth:`accept` method takes value from field
    and converts it to python type.

    Subclasses must redefine :meth:`to_python` method to make their 
    own convertation and validation.

    :meth:`from_python` method takes value as python
    object and converts it to string or something
    else widget can display.

    Filters and validators:

    Filters are functions performing additional validation and convertation 
    after :meth:`to_python` method. The interface of filters is following::

        def filter_value(conv, value):
            if wrong(value):
                raise ValidationError(..)
            new_value = do_smth(value)
            return new_value

        convs.Char(filter_value, required=True)

    Validators are shortcuts to filters that do no convertations, but  only
    do assertions::

        @validator(error_message)
        def validate(conv, value):
            return is_valid(value)

    Both filters and validators can be passed to converter as positional 
    arguments and will be applied after :meth:`to_python` method and 
    `required` check in order they are mentioned.

    Error messages redefinition:

    Set `error_<type>` parameter to your own message template, for example::

        convs.Char(regex_readable="YY.MM.DD",
                   error_regex='Should match %(regex_readable)s')`
    '''

    # obsolete parameters from previous versions
    _obsolete = frozenset(['max_length', 'min_length', 'null', 'min', 'max',
                           'multiple', 'initial'])
    required = False
    multiple = False

    validators = ()
    #: Values are not accepted by Required validator
    error_required = N_('required field')

    def __init__(self, *args, **kwargs):
        if self._obsolete & set(kwargs):
            raise TypeError(
                    'Obsolete parameters are used: %s' %
                        list(self._obsolete & set(kwargs)))
        self.field = weakproxy(kwargs.get('field'))
        self._init_kwargs = kwargs
        self.__dict__.update(kwargs)
        self.validators = tuple(self.validators) + args

    @property
    def env(self):
        return self.field.env

    def _is_empty(self, value):
        return value in ('', [], {}, None)

    def accept(self, value, silent=False):
        '''
        Accepts a value from the form, calls :meth:`to_python` method,
        checks `required` condition, applies filters and validators,
        catches ValidationError.

        If `silent=False`, writes errors to `form.errors`.
        '''
        try:
            value = self.to_python(value)
            for v in self.validators:
                value = v(self, value)

            if self.required and self._is_empty(value):
                raise ValidationError(self.error_required)
        except ValidationError, e:
            if not silent:
                e.fill_errors(self.field)
            #NOTE: by default value for field is in python_data,
            #      but this is not true for FieldList where data
            #      is dynamic, so we set value to None for absent value.
            value = self._existing_value
        return value

    def to_python(self, value):
        """
        Converts value and validates it.
        Custom converters should override this
        """
        if value == '':
            return None # XXX is this right?
        return value

    def from_python(self, value):
        """
        Serializes value.
        Custom converters should override this
        """
        if value is None:
            value = ''
        return value

    def __call__(self, *args, **kwargs):
        '''
        Creates current object's copy with extra constructor arguments
        (including validators) passed.
        '''
        kwargs = dict(self._init_kwargs, **kwargs)
        kwargs.setdefault('field', self.field)
        validators = tuple(self.validators) + args
        return self.__class__(*validators, **kwargs)

    def assert_(self, expression, msg):
        'Shortcut for assertions of certain type'
        if not expression:
            raise ValidationError(msg)

    @property
    def _existing_value(self):
        if self.field is not None:
            return self.field.parent.python_data.get(self.field.name)
        return [] if self.multiple else None

    def __repr__(self):
        args = ', '.join([k+'='+repr(v) for
                          k, v in self._init_kwargs.items()
                          if k!='parent'])
        return '{}({})'.format(self.__class__.__name__, args)


class validator(object):
    'Function decorator'
    def __init__(self, message):
        self.message = message
    def __call__(self, func):
        def wrapper(conv, value):
            if not func(conv, value):
                raise ValidationError(self.message)
            return value
        return wrapper

# Some useful validators

def length(min_length, max_length):
    'Sting length constraint'
    if min_length == max_length:
        message = M_(u'length of value must be exactly %(max)d symbol',
                     u'length of value must be exactly %(max)d symbols',
                     count_field="max")
    else:
        message = N_('length should be between %(min)d and %(max)d symbols')

    message = message % dict(min=min_length, max=max_length)

    @validator(message)
    def wrapper(conv, value):
        if not value:
            # it meens that this value is not required
            return True
        if len(value) < min_length:
            return False
        if len(value) > max_length:
            return False
        return True
    return wrapper


@deprecated('Use length(min, max) instead.')
def limit(min_length, max_length):
    return length(min_length, max_length) # pragma: no cover


def between(min_value, max_value):
    'Numerical values limit'
    message = N_('value should be between %(min)d and %(max)d') % \
                    dict(min=min_value, max=max_value)

    @validator(message)
    def wrapper(conv, value):
        if value is None:
            # it meens that this value is not required
            return True
        if value < min_value:
            return False
        if value > max_value:
            return False
        return True
    return wrapper

@deprecated('Use between(min, max) instead.')
def num_limit(min_value, max_value):
    return between(min_value, max_value) # pragma: no cover


class CharBased(Converter):

    nontext_replacement = u'\uFFFD' # Set None to disable and empty string to
                                    # remove.
                                    # Default value is u"�"
    strip = True

    def clean_value(self, value):
        '''
        Additional clean action to preprocess value before :meth:`to_python`
        method.

        Subclasses may define own clean_value method to allow additional clean
        actions like html cleanup, etc.
        '''
        # We have to clean before checking min/max length. It's done in
        # separate method to allow additional clean action in subclasses.
        if self.nontext_replacement is not None:
            value = replace_nontext(value, self.nontext_replacement)
        if self.strip:
            value = value.strip()
        return value


class Char(CharBased):

    '''Converts and validates strings'''

    #: Regexp to match input string
    regex = None

    error_regex = N_('field should match %(regex)s')

    def to_python(self, value):
        # converting
        value = self.clean_value(value)
        if value and self.regex:
            regex = self.regex
            if isinstance(self.regex, basestring):
                regex = re.compile(self.regex, re.U)
            if not regex.match(value):
                error = self.error_regex % {'regex': self.regex}
                raise ValidationError(error)
        return value

    def from_python(self, value):
        if value is None:
            return ''
        return unicode(value)


class Int(Converter):
    """
    Converts digital sequences to `int'
    """

    error_notvalid = N_('it is not valid integer')

    def to_python(self, value):
        if value == '':
            return None
        try:
            value = int(value)
        except ValueError:
            raise ValidationError(self.error_notvalid)
        return value

    def from_python(self, value):
        if value is None:
            return ''
        return unicode(value)


class Bool(Converter):
    """
    Converts to True/False
    """
    required = False

    def to_python(self, value):
        return bool(value)

    def from_python(self, value):
        if value:
            return 'checked'
        return ''


class DisplayOnly(Converter):

    def from_python(self, value):
        return value

    def to_python(self, value):
        return self._existing_value


class EnumChoice(Converter):
    '''In addition to Converter interface it must provide methods options and
    get_label.'''

    conv = Char()
    # choices: [(python_value, label), ...]
    choices = ()
    error_required = N_('you must select a value')

    def from_python(self, value):
        conv = self.conv
        return conv.from_python(value)

    def to_python(self, value):
        value = self.conv.accept(value, silent=True)
        if value not in dict(self.choices):
            return None
        return value

    def options(self):
        conv = self.conv
        for python_value, label in self.choices:
            yield conv.from_python(python_value), label

    def get_label(self, value):
        # XXX comment needed
        value = self.conv.accept(value, silent=True)
        return dict(self.choices).get(value)


class BaseDatetime(CharBased):

    format = None
    readable_format = None
    replacements = (('%H', 'HH'), ('%M', 'MM'), ('%d', 'DD'),
                    ('%m', 'MM'), ('%Y', 'YYYY'))
    error_wrong_format = N_('Wrong format (%(readable_format)s)')
    nontext_replacement = ''

    def __init__(self, *args, **kwargs):
        if not 'readable_format' in kwargs or 'format' in kwargs:
            replacements = self.replacements # XXX make language-dependent
            fmt = kwargs.get('format', self.format)
            for repl in replacements:
                fmt = fmt.replace(*repl)
            kwargs['readable_format'] = fmt
        Converter.__init__(self, *args, **kwargs)

    def from_python(self, value):
        if value is None:
            return ''
        # carefull to years before 1900
        return strftime(value, self.format)

    def to_python(self, value):
        value = self.clean_value(value)
        if not value:
            return None
        try:
            return self.convert_datetime(value)
        except ValueError:
            raise ValidationError(self.error_wrong_format)


class Datetime(BaseDatetime):

    format = '%d.%m.%Y, %H:%M'

    def convert_datetime(self, value):
        return datetime.strptime(value, self.format)


class Date(BaseDatetime):

    format = '%d.%m.%Y'

    def convert_datetime(self, value):
        return datetime.strptime(value, self.format).date()


class Time(BaseDatetime):

    format = '%H:%M'

    def from_python(self, value):
        if value is None:
            return ''
        # we don't care about year in time converter, so use native strftime
        return value.strftime(self.format)

    def convert_datetime(self, value):
        return datetime.strptime(value, self.format).time()


class SplitDateTime(Converter):

    def from_python(self, value):
        if value is None:
            return {'date': None, 'time': None}
        else:
            return {'date': value.date(), 'time': value.time()}

    def to_python(self, value):
        if value['date'] is None or value['time'] is None:
            return None
        res = datetime.combine(value['date'], value['time'])
        return res


class Html(Char):
    '''
    Converter for flexible cleanup of HTML document fragments.
    A subclass of :class:`Char<iktomi.forms.convs.Char>`.

    Uses :class:`utils.html.Sanitizer<iktomi.utils.html.Sanitizer>`
    instance to sanitize input HTML.

    Construtor collects from given kwargs all of
    :class:`Sanitizer<iktomi.utils.html.Sanitizer>`
    options and passes them into Sanitizer's constructor.

    For list properties it is allowed to use :meth:`add_%s` interface::

        Html(add_allowed_elements=['span'], add_dom_callbacks=[myfunc])
    '''

    allowed_elements = frozenset(('a', 'p', 'br', 'li', 'ul', 'ol', 'hr', 'u',
                                  'i', 'b', 'blockquote', 'sub', 'sup'))
    allowed_attributes = frozenset(('href', 'src', 'alt', 'title', 'class', 'rel'))
    drop_empty_tags = frozenset(('p', 'a', 'u', 'i', 'b', 'sub', 'sup'))
    allowed_protocols = frozenset(['ftp', 'http', 'https', 'mailto',
                                   'tel', 'webcal', 'callto'])
    allowed_classes = {}
    dom_callbacks = []
    #: Function returning object marked safe for template engine.
    #: For example: jinja Markup object
    Markup = lambda s, x: x
    Cleaner = Cleaner
    class Nothing: pass

    PROPERTIES = ['allowed_elements', 'allowed_attributes', 'allowed_protocols',
                  'allowed_classes', 'dom_callbacks', 'drop_empty_tags']

    LIST_PROPERTIES = ['allowed_elements', 'allowed_attributes',
                       'allowed_protocols', 'dom_callbacks', 'drop_empty_tags']

    @classmethod
    def _load_arg(cls, opt):
        assert hasattr(cls, opt)
        # XXX very complicated merges for rare cases
        #     not sure if they are necessary
        if opt in cls.__dict__:
            result = getattr(cls, opt)
        else:
            for base in cls.__bases__:
                if hasattr(base, '_load_arg'):
                    result = base._load_arg(opt)
                    if not result is cls.Nothing:
                        break
            else:
                result = cls.Nothing

        if 'add_' + opt in cls.__dict__:
            add = getattr(cls, 'add_' + opt)
            if result is cls.Nothing:
                result = add
            else:
                result = set(result)
                result.update(add)
        return result

    def __init__(self, *args, **kwargs):
        assert self.Cleaner is not None, \
                'Install lxml or implement your own html cleaner'
        for opt in self.PROPERTIES:
            if not opt in kwargs:
               result = self._load_arg(opt)
               if not result is self.Nothing:
                   kwargs[opt] = result

        for opt in self.LIST_PROPERTIES:
            add_key = 'add_' + opt
            if add_key in kwargs:
                opt_value = kwargs.get(opt, [])
                # XXX sometimes set must be ordered
                kwargs[opt] = set(opt_value)
                kwargs[opt].update(kwargs.pop(add_key))

        Char.__init__(self, *args, **kwargs)

    def clean_value(self, value):
        value = Char.clean_value(self, value)
        try:
            doc = html.fragment_fromstring(value, create_parent=True)
        except XMLSyntaxError:
            raise ValidationError(N_(u'Error parsing HTML'))
        self.cleaner(doc)
        clean = html.tostring(doc, encoding='utf-8').decode('utf-8')
        clean = clean.split('>', 1)[1].rsplit('<', 1)[0]
        return self.Markup(clean)

    @cached_property
    def cleaner(self):
        return self.Cleaner(allow_tags=self.allowed_elements,
                            safe_attrs=self.allowed_attributes,
                            allow_classes=self.allowed_classes,
                            allowed_protocols=self.allowed_protocols,
                            drop_empty_tags=self.drop_empty_tags,
                            dom_callbacks=self.dom_callbacks,
                            )


class List(Converter):
    '''
    Converter for FieldList'''

    _obsolete = Converter._obsolete | set(['filter'])

    def from_python(self, value):
        result = OrderedDict()
        for index, item in enumerate(value):
            result[str(index+1)] = item
        return result

    def to_python(self, value):
        return value.values()


class ListOf(Converter):
    '''
    Converter for scalar Fields, applies nested converter to each value
    of the field (i.e. for each value from MultiDict) and returns a list of 
    resulting values.

    Usage::

        ListOf(Converter(), *validators, **kwargs)
    '''

    multiple = True

    def __init__(self, *args, **kwargs):
        if args and (isinstance(args[0], Converter) or \
                     (isinstance(args[0], type) and \
                      issubclass(args[0], Converter))):
            conv = args[0]
            args = args[1:]
        else:
            conv = kwargs['conv']
        if 'field' in kwargs:
            conv = conv(field=kwargs['field'])
        kwargs['conv'] = conv
        Converter.__init__(self, *args, **kwargs)

    def to_python(self, value):
        result = []
        for val in value or []:
            val = self.conv.accept(val)
            if val is not None:
                # XXX is it right to ignore None?
                result.append(val)
        return result

    def from_python(self, value):
        return [self.conv.from_python(item) for item in value or []]


class FieldBlockConv(Converter):

    @property
    def _existing_value(self):
        if self.field is not None:
            return self.field.python_data
        return {} # XXX


class SimpleFile(Converter):

    def _is_empty(self, file):
        return file == u'' or file is None #XXX WEBOB ONLY !!!

    def to_python(self, file):
        if not self._is_empty(file):
            return file

    def from_python(self, value):
        return None

# Expose all variables defined after imports
__all__ = [x for x
           in set(locals().keys()) - set(_all2)
           if not x.startswith('_')]
del _all2




# -*- coding: utf-8 -*-

import sys
import datetime


__all__ = ['manage']


class CommandNotFound(AttributeError): pass
class ConverterError(Exception): pass


def manage(commands, argv=None, delim=':'):
    '''
    Parses argv and runs neccessary command. Is to be used in manage.py file.

    Accept a dict with digest name as keys and instances of
    :class:`Cli<iktomi.management.commands.Cli>`
    objects as values.

    The format of command is the following::

        ./manage.py digest_name:command_name[ arg1[ arg2[...]]][ --key1=kwarg1[...]]

    where command_name is a part of digest instance method name, args and kwargs
    are passed to the method. For details, see
    :class:`Cli<iktomi.management.commands.Cli>` docs.
    '''
    argv = sys.argv if argv is None else argv
    if len(argv) > 1:
        cmd_name = argv[1]
        raw_args = argv[2:]
        args, kwargs = [], {}
        # parsing params
        for item in raw_args:
            if item.startswith('--'):
                splited = item[2:].split('=', 1)
                if len(splited) == 2:
                    k,v = splited
                elif len(splited) == 1:
                    k,v = splited[0], True
                else:
                    sys.exit('Error while parsing argument "%s"' % item)
                kwargs[k] = v
            else:
                args.append(item)

        # trying to get command instance
        if delim in cmd_name:
            digest_name, command = cmd_name.split(delim)
        else:
            digest_name = cmd_name
            command = None
        try:
            digest = commands[digest_name]
        except KeyError:
            _command_list(commands)
            sys.exit('Command "%s" not found' % digest_name)
        try:
            if command is None:
                if isinstance(digest, Cli):
                    help_ = digest.description(argv[0], digest_name)
                    sys.stdout.write(help_)
                    sys.exit('ERROR: "%s" is command digest' % digest_name)
                    return
                digest(*args, **kwargs)
            else:
                digest(command, *args, **kwargs)
        except CommandNotFound:
            help_ = digest.description(argv[0], digest_name)
            sys.stdout.write(help_)
            sys.exit('Command "%s:%s" not found' % (digest_name, command))
    else:
        _command_list(commands)
        sys.exit('Please provide any command')

def _command_list(commands):
    sys.stdout.write('Commands:\n')
    for k in commands.keys():
        sys.stdout.write(str(k))
        sys.stdout.write('\n')


class Cli(object):
    ''

    def description(self, argv0='manage.py', command=None):
        '''Description outputed to console'''
        command = command or self.__class__.__name__.lower()

        import inspect
        _help = ''
        if self.__doc__:
            _help += '%s\n' % self.__doc__
        else:
            _help += '%s\n' % command
        for attr in dir(self):
            if attr.startswith('command'):
                func = getattr(self, attr)
                if func.__doc__:
                    _help += "\t%s\n" % func.__doc__
                else:
                    comm = attr.replace('command_', '', 1)
                    args = inspect.getargspec(func).args[1:]
                    args = (' [' + '] ['.join(args) + ']') if args else ''

                    _help += "\t./%s %s:%s%s\n" % \
                                     (argv0, command, comm, args)
        return _help

    def __call__(self, command_name, *args, **kwargs):
        if command_name == 'help':
            sys.stdout.write(self.description())
        elif hasattr(self, 'command_'+command_name):
            try:
                getattr(self, 'command_'+command_name)(*args, **kwargs)
            except ConverterError, e:
                sys.stderr.write('One of the arguments for '
                                 'command "%s" is wrong:\n' % command_name)
                sys.stderr.write(str(e))
        else:
            raise CommandNotFound()


class argument(object):
    'Decorator. Helps to define desired argument types'

    def __init__(self, arg_id, *validators, **kwargs):
        assert type(arg_id) in (int, str), \
            'First argument of argument (%r) decorator must be "str" or "int" type' % arg_id
        self.arg_id = arg_id
        self.validators = validators
        self.required = kwargs.get('required', True)

    def __call__(self, func):
        def wrapper(*args, **kwargs):
            args = list(args)
            if isinstance(self.arg_id, int):
                try:
                    arg = args[self.arg_id]
                except IndexError:
                    raise ConverterError('Total positional args = %d, '
                        'but you apply converter for %d argument '
                        '(indexing starts from 0)' % (len(args), self.arg_id))
                arg = self.convert(arg)
                args[self.arg_id] = arg
            elif isinstance(self.arg_id, str):
                arg = kwargs.get(self.arg_id)
                if arg:
                    arg = self.convert(arg)
                    kwargs[self.arg_id] = arg
                if not arg and self.required:
                    raise ConverterError('Keyword argument "%s" is required' % self.arg_id)
            return func(*args, **kwargs)
        return wrapper

    def convert(self, arg):
        for v in self.validators:
            arg = v(arg)
        return arg

    @staticmethod
    def to_int(value):
        try:
            return int(value)
        except Exception:
            raise ConverterError('Cannot convert %r to int' % value)

    @staticmethod
    def to_date(value):
        try:
            return datetime.datetime.strptime(value, '%d/%m/%Y').date()
        except Exception:
            raise ConverterError('Cannot convert %r to date, please provide '
                                 'string in format "dd/mm/yyyy"' % value)



import warnings
from sqlalchemy.orm.collections import InstrumentedList

warnings.warn('OrderedList is deprecated. Use '\
                'sqlalchemy.ext.orderinglist.ordering_list() instead',
              category=DeprecationWarning)


class OrderedList(InstrumentedList):
    '''This collection automatically updates order column to preserve original
    order of items when they are loaded with corresponding ORDER BY clause.'''

    order_column = 'order'

    def append(self, item):
        if self:
            order = getattr(self[-1], self.order_column)+1
        else:
            order = 1
        setattr(item, self.order_column, order)
        InstrumentedList.append(self, item)



# -*- coding: utf-8 -*-
from sqlalchemy.ext.associationproxy import _AssociationCollection
from iktomi.forms.convs import *
from iktomi.forms.convs import __all__ as _all1

import re
from iktomi.utils import N_

_all2 = locals().keys()


class Email(Char):

    regex = re.compile(
        # dot-atom
        r"(^[-!#$%&'*+/=?^_`{}|~0-9A-Z]+(\.[-!#$%&'*+/=?^_`{}|~0-9A-Z]+)*"
        # Although quoted variant is allowed by spec it's actually not used
        # except by geeks that are looking for problems. But the characters
        # allowed in quoted string are not safe for HTML and XML, so quoted
        # e-mail can't be expressed in such formats directly which is quite
        # common. We prefer to forbid such valid but unsafe e-mails to avoid
        # security problems. To allow quoted names disable non-text characters
        # replacement and uncomment the following lines of regexp:
        ## quoted-string
        #r'|^"([\001-\010\013\014\016-\037!#-\[\]-\177]|'
        #    r'\\[\001-011\013\014\016-\177])*"'
        r')@(?:[A-Z0-9-]+\.)+[A-Z]{2,6}$', re.IGNORECASE)
    error_regex = N_('incorrect e-mail address')


class ModelDictConv(Converter):
    '''Converts a dictionary to object of `model` class with the same fields.
    It is designed for use in FieldSet'''

    model = None

    def from_python(self, value):
        if value is None:
            # Field set can be optional
            return {}
        result = {}
        field_names = sum([x.field_names for x in self.field.fields], [])
        for field_name in field_names:
            attr = getattr(value, field_name)
            if issubclass(attr.__class__, _AssociationCollection):
                attr = attr.copy()
            result[field_name] = attr
        return result

    def to_python(self, value):
        obj = self.model()
        field_names = sum([x.field_names for x in self.field.fields], [])
        for field_name in field_names:
            field = self.field.get_field(field_name)
            if 'w' in field.permissions:
                setattr(obj, field_name, value[field_name])
        return obj

    @property
    def _existing_value(self):
        if self.field is not None:
            pd = self.field.parent.python_data
            if self.field.name in pd:
                return pd[self.field.name]
            # Return blank self.model instance as initial/default value 
            # if one does not exist
            return self.model()
        return [] if self.multiple else None


class OptionLabel(unicode):

    published = False


class ModelChoice(EnumChoice):

    condition = None
    conv = Int(required=False)
    title_field = 'title'

    def __init__(self, *args, **kwargs):
        EnumChoice.__init__(self, *args, **kwargs)
        self.conv = self.conv(field=self.field)

    @property
    def query(self):
        query = self.env.db.query(self.model)
        if isinstance(self.condition, dict):
            query = query.filter_by(**self.condition)
        elif self.condition is not None:
            query = query.filter(self.condition)
        return query

    def from_python(self, value):
        if value is not None:
            return self.conv.from_python(value.id)
        else:
            return ''

    def to_python(self, value):
        try:
            value = self.conv.to_python(value)
        except ValidationError:
            return None
        else:
            if value is not None:
                return self.query.filter_by(id=value).first()

    def get_object_label(self, obj):
        label = OptionLabel(getattr(obj, self.title_field))
        try:
            label.published = obj.publish
        except AttributeError:
            pass
        return label

    def get_label(self, form_value):
        obj = self._safe_to_python(form_value)
        if obj is not None:
            return self.get_object_label(obj)

    def options(self):
        for obj in self.query.all():
            yield self.conv.from_python(obj.id), self.get_object_label(obj)


# Expose all variables defined after imports and all variables imported from
# parent module
__all__ = [x for x
           in set(locals().keys()) - (set(_all2) - set(_all1))
           if not x.startswith('_')]
del _all1, _all2



# -*- coding: utf-8 -*-

import sys
from sqlalchemy import create_engine
from sqlalchemy.types import SchemaType
from sqlalchemy.engine import reflection
from sqlalchemy.schema import (
    MetaData,
    Table,
    DropTable,
    ForeignKeyConstraint,
    DropConstraint,
)
from .base import Cli

__all__ = ['Sqla']


def drop_everything(engine):
    '''Droping all tables and custom types (enums) using `engine`.
    Taken from http://www.sqlalchemy.org/trac/wiki/UsageRecipes/DropEverything
    
    This method is more robust than `metadata.drop_all(engine)`. B.c. when
    you change a table or a type name, `drop_all` does not consider the old one.
    Thus, DB holds some unused entities.'''
    conn = engine.connect()
    # the transaction only applies if the DB supports
    # transactional DDL, i.e. Postgresql, MS SQL Server
    trans = conn.begin()
    inspector = reflection.Inspector.from_engine(engine)
    metadata = MetaData()
    tbs = []
    all_fks = []
    types = []
    for table_name in inspector.get_table_names():
        fks = []
        for fk in inspector.get_foreign_keys(table_name):
            if not fk['name']:
                continue
            fks.append(ForeignKeyConstraint((), (), name=fk['name']))
        for col in inspector.get_columns(table_name):
            if isinstance(col['type'], SchemaType):
                types.append(col['type'])
        t = Table(table_name,metadata,*fks)
        tbs.append(t)
        all_fks.extend(fks)
    try:
        for fkc in all_fks:
            conn.execute(DropConstraint(fkc))
        for table in tbs:
            conn.execute(DropTable(table))
        for custom_type in types:
            custom_type.drop(conn)
        trans.commit()
    except:
        trans.rollback()
        raise


class Sqla(Cli):
    'SQLAlchemy database handling'

    def __init__(self, session_maker, initial=None, generators=None):
        self.session = session_maker()
        self.initial = initial
        self.generators = generators or {}

    def _get_binds(self):
        metadatas = {}
        for table, engine in self.session._Session__binds.items():
            metadatas.setdefault(table.metadata, set()).add(engine)
        return metadatas

    def _schema(self, table):
        from sqlalchemy.schema import CreateTable
        return str(CreateTable(table))

    def command_create_tables(self):
        print('Creating table(s)...')
        for metadata, engines in self._get_binds().items():
            for engine in engines:
                for table in metadata.sorted_tables:
                    # XXX Output of schema is commented since it doesn't work
                    # for dialect-specific things.
                    #print('{0}: {1}\n{2}'.format(engine, table.name, self._schema(table)))
                    metadata.create_all(engine, tables=[table])

    def command_drop_tables(self):
        answer = raw_input('All data will lost. Are you sure? [y/N] ')
        if answer.strip().lower()!='y':
            sys.exit('Interrupted')
        print('Droping table(s)...')
        for metadata, engines in self._get_binds().items():
            for engine in engines:
                print('... in {0}'.format(engine.url))
                drop_everything(engine)
        print('Done')

    def command_init(self):
        if self.initial:
            self.initial(self.session)

    def command_reset(self):
        self.command_drop_tables()
        self.command_create_tables()
        self.command_init()

    def command_schema(self, model_name=None):
        for table, engine in self.session._Session__binds.items():
            if model_name:
                if model_name == table.name:
                    print(self._schema(table))
            else:
                print(self._schema(table))

    def command_gen(self, *names):
        if not names:
            raise Exception('Please provide generator names')
        for name in names:
            name, count = name, 0
            if ':' in name:
                name, count = name.split(':', 1)
            count = int(count)
            create = self.generators[name]
            print('Generating `{0}` count={1}'.format(name, count))
            create(self.session, count)
            self.session.commit()



# -*- coding: utf-8 -*-

from distutils.core import setup

setup(
    name='iktomi',
    version='0.4',
    packages=['iktomi',
              'iktomi.utils',
              'iktomi.forms',
              'iktomi.web',
              'iktomi.templates',
                    'iktomi.templates.mint', 'iktomi.templates.jinja2',
              'iktomi.db',
                    'iktomi.db.sqla',
              'iktomi.cli',
              'iktomi.unstable',
                    'iktomi.unstable.forms', 'iktomi.unstable.web',
                    'iktomi.unstable.utils',
                    'iktomi.unstable.db',
                        'iktomi.unstable.db.sqla'],
    package_dir={
        'iktomi.templates.jinja2': 'iktomi/templates/jinja2',
        'iktomi.templates.mint': 'iktomi/templates/mint',
    },
    package_data={
        'iktomi.templates.jinja2': ['templates/*/*.html'],
        'iktomi.templates.mint': ['templates/*/*.mint'],
    },
    requires=[
        'webob (>1.1b1)',
    ],
    author='Denis Otkidach',
    author_email='denis.otkidach@gmail.com',
    maintainer='Harut Dagesyan',
    maintainer_email='yes@harutune.name',
    description='A web tool: routing, forms, other useful things.',
    #long_description=open('README').read(),
    url='http://github.com/SmartTeleMax/iktomi/',
    license='MIT',
    keywords='web forms',
)



# -*- coding: utf-8 -*-

__all__ = ['URL']

import urllib
from urlparse import urlparse, parse_qs
from webob.multidict import MultiDict
from .url_templates import urlquote


def construct_url(path, query, host, port, schema):
    query = ('?' + '&'.join(['%s=%s' % (urlquote(k), urlquote(v)) \
                            for k,v in query.iteritems()])  \
             if query else '')

    path = path
    if host:
        host = host.encode('idna')
        port = ':' + port if port else ''
        return ''.join((schema, '://', host, port, path,  query))
    else:
        return path + query


class URL(str):

    def __new__(cls, path, query=None, host=None, port=None, schema=None, show_host=True):
        '''
        path - urlencoded string or unicode object (not encoded at all)
        '''
        path = path if isinstance(path, str) else urlquote(path)
        query = MultiDict(query) if query else MultiDict()
        host = host or ''
        port = port or ''
        schema = schema or 'http'
        self = str.__new__(cls, construct_url(path, query, host if show_host else '', port,schema))
        self.path = path
        self.query = query
        self.host = host
        self.port = port
        self.schema = schema
        self.show_host = show_host
        return self

    @classmethod
    def from_url(cls, url, show_host=True):
        '''Parse string and get URL instance'''
        # url must be idna-encoded and url-quotted
        url = urlparse(url)
        query = sum([[(k.decode('utf-8'), v.decode('utf-8'))
                      for v in values]
                     for k, values in parse_qs(url.query).items()], [])
        host = url.netloc.split(':', 1)[0] if ':' in url.netloc else url.netloc
        port = url.netloc.split(':')[1] if ':' in url.netloc else ''
        return cls(urllib.unquote(url.path).decode('utf-8'),
                   query, host.decode('idna'),
                   port, url.scheme, show_host)

    def _copy(self, **kwargs):
        path = kwargs.pop('path', self.path)
        kw = dict(query=self.query, host=self.host, 
                  port=self.port, schema=self.schema,
                  show_host=self.show_host)
        kw.update(kwargs)
        return self.__class__(path, **kw)

    def qs_set(self, *args, **kwargs):
        '''Set values in QuerySet MultiDict'''
        if args and kwargs:
            raise TypeError('Use positional args or keyword args not both')
        query = self.query.copy()
        if args:
            mdict = MultiDict(args[0])
            for k in mdict.keys():
                if k in query:
                    del query[k]
            for k, v in mdict.items():
                query.add(k, v)
        else:
            for k, v in kwargs.items():
                query[k] = v
        return self._copy(query=query)

    def qs_add(self, *args, **kwargs):
        '''Add value to QuerySet MultiDict'''
        query = self.query.copy()
        if args:
            mdict = MultiDict(args[0])
            for k, v in mdict.items():
                query.add(k, v)
        for k, v in kwargs.items():
            query.add(k, v)
        return self._copy(query=query)

    def with_host(self):
        '''Force show_host parameter'''
        return self._copy(show_host=True)

    def qs_delete(self, key):
        '''Delete value from QuerySet MultiDict'''
        query = self.query.copy()
        try:
            del query[key]
        except KeyError:
            pass
        return self._copy(query=query)

    def qs_get(self, key, default=None):
        '''Get a value from QuerySet MultiDict'''
        return self.query.get(key, default=default)

    def get_readable(self):
        '''Gets human-readable representation of the url (as unicode string)'''
        query = (u'?' + u'&'.join([u'%s=%s' % (k,v) for k, v in self.query.iteritems()]) \
                 if self.query else '')

        path = urllib.unquote(self.path).decode('utf-8')
        if self.host:
            port = u':' + self.port if self.port else u''
            return u''.join((self.schema, '://', self.host, port, path,  query))
        else:
            return path + query

    def __repr__(self):
        return '<URL %r>' % str.__repr__(self)



# -*- coding: utf-8 -*-

import os
import sys
import time
import logging
import threading
from os import path
from itertools import chain

from .base import Cli, CommandNotFound

__all__ = ['App']


logger = logging.getLogger(__name__)


try:
    MAXFD = os.sysconf("SC_OPEN_MAX")
except:
    MAXFD = 256


def close_fds(but=None):
    if but is None:
        os.closerange(3, MAXFD)
        return
    os.closerange(3, but)
    os.closerange(but + 1, MAXFD)


def flush_fds():
    for fd in range(3, MAXFD + 1):
        try:
            os.fsync(fd)
        except OSError:
            pass


class App(Cli):
    'Development application'
    format = '%(levelname)s [%(name)s] %(message)s'

    def __init__(self, app, shell_namespace=None, extra_files=None, bootstrap=None):
        self.app = app
        self.shell_namespace = shell_namespace or {}
        self.extra_files = extra_files
        self.bootstrap = bootstrap

    def command_serve(self, host='', port='8000', level='debug'):
        logging.basicConfig(level=getattr(logging, level.upper()), format=self.format)
        if self.bootstrap:
            logger.info('Bootstraping...')
            self.bootstrap()
        try:
            server_thread = DevServerThread(host, port, self.app)
            server_thread.start()

            wait_for_code_change(extra_files=self.extra_files)

            server_thread.running = False
            server_thread.join()
            logger.info('Reloading...')

            # Smart reload of current process.
            # Main goal is to reload all modules
            # NOTE: For exec syscall we need to flush and close all fds manually
            flush_fds()
            close_fds()
            os.execvp(sys.executable, [sys.executable] + sys.argv)
        except KeyboardInterrupt:
            logger.info('Stoping dev-server...')
            server_thread.running = False
            server_thread.join()
            sys.exit()

    def command_shell(self):
        from code import interact
        interact('Namespace %r' % self.shell_namespace,
                 local=self.shell_namespace)


class DevServerThread(threading.Thread):

    def __init__(self, host, port, app):
        from wsgiref.simple_server import make_server, WSGIServer, \
                WSGIRequestHandler
        self.host = host
        self.port = port

        class DevServer(WSGIServer):
            timeout = 0.2


        class RequestHandler(WSGIRequestHandler):
            def address_string(slf):
                # getfqdn sometimes is very slow
                return '%s:%s' % (host, port)

            def log_message(self, format, *args):
                logger.info("%s - - [%s] %s",
                            self.client_address[0],
                            self.log_date_time_string(),
                            format%args)

        try:
            self.port = int(port)
        except ValueError:
            raise ValueError('Please provide valid port value insted of "%s"' % port)
        self.running = True
        self.server = make_server(self.host, self.port, app, server_class=DevServer,
                                  handler_class=RequestHandler)
        super(DevServerThread, self).__init__()

    def run(self):
        logger.info('Devserver is running on port %s\n', self.port)
        while self.running:
            self.server.handle_request()


# All reloader utils are taken from werkzeug
def iter_module_files():
    for module in sys.modules.values():
        filename = getattr(module, '__file__', None)
        if filename:
            while not os.path.isfile(filename):
                filename = os.path.dirname(filename)
                if not filename:
                    break
            else:
                if filename[-4:] in ('.pyc', '.pyo'):
                    filename = filename[:-1]
                yield filename


def wait_for_code_change(extra_files=None, interval=1):
    mtimes = {}
    while 1:
        for filename in chain(iter_module_files(), extra_files or ()):
            try:
                mtime = os.stat(filename).st_mtime
            except OSError:
                continue

            old_time = mtimes.get(filename)
            if old_time is None:
                mtimes[filename] = mtime
            elif mtime > old_time:
                logger.info('Changes in file "%s"', filename)
                return
        time.sleep(interval)

